Program Ping;

uses
    Crt, SysUtils, Network, PktDrv, IP, ICMP, DNS, Util;

Const
    MY_IP          	= '128.128.1.217';
    MY_NETMASK		= '255.255.255.0';
    MY_GATEWAY		= '128.128.1.100';
    DNS_SERVER		= '128.128.1.100';

Var
    Dest: TIPAddress;
    IPHandle: Integer;
    IPConf: TIPAdapterConfig;
    Adapter: TPacketDriverAdapter;
begin

    // Check Params
    if ParamCount <> 1 then begin
      	WriteLn('Usage: PING destination');
        WriteLn('       Where destination is an IP Address or Host Name.');
        Halt(1);
    end;

    // Create an Adapter object representing out network adapter.
    Adapter := TPacketDriverAdapter.Create;

    // If the adapter works fine, we bind the TCP/IP protocol.
    if Adapter.Status <> asOk then begin
        WriteLn('Could not initialise network adapter. PKTPM is not loaded.');
		Halt(2);
    end;
    IPConf.DHCP      := False;
    IPConf.IPAddress := IP_StringToAddr(MY_IP);
    IPConf.NetMask   := IP_StringToAddr(MY_NETMASK);
    IPConf.Gateway   := IP_StringToAddr(MY_GATEWAY);
    IPHandle         := IP_Init(Adapter, IPConf);
    if IPHandle = 0 then begin
        WriteLn('Could not initialise IP stack. Wrong Parameters.');
        Adapter.Free;
        Halt(3);
    end;
    SetDNSServer(IP_StringToAddr(DNS_SERVER), Swap(WORD(53)));

    // Destination is a name or an IP address?
    Dest := IP_StringToAddr(ParamStr(1));
    if Dest = 0 then begin
    	// Doesn't seem to be an IP. Let's try to resolve it with DNS.
        if not GetHostByName(ParamStr(1), Dest) then begin
            WriteLn('Could not resolve host name "' + ParamStr(1) + '".');
            IP_Deinit(IPHandle);
            Adapter.Free;
            Halt(4);
        end;
    end;

    // Ping until <Esc> pressed
    Repeat
        Writeln('Pinging to ' + IP_AddrToString(Dest) + '...');
        if ICMP_Ping(Dest, 2000) = 0 then begin
		    WriteLn('Response received from ' + IP_AddrToString(Dest));
	    	Delay(1000);
        end else begin
            WriteLn('Timeout');
        end;
    Until KeyPressed and (ReadKey = #27);

    // Close everything
    IP_Deinit(IPHandle);
    Adapter.Free;
end.