Unit ARP;

interface

Uses
    Network, Crt, Objects, SysUtils;

Type
    { Routine for servicing ARP packets }
    TARP_Routine = procedure(Adapter: TNetworkAdapter;
	  SourceEtherAddr: TEthernetAddress;
	  Operation: WORD; PLEN: BYTE; SenderHA: TEthernetAddress;
	  SenderPA: Pointer; TargetHA: TEthernetAddress; TargetPA: Pointer);

Function  ARP_AddProcessor(Adapter: TNetworkAdapter; ProtocolType: WORD; Routine: TARP_Routine): Boolean;
Function  ARP_RemoveProcessor(Adapter: TNetworkAdapter; ProtocolType: WORD): Boolean;
Function  ARP_Send(Adapter: TNetworkAdapter; DestEtherAddr: TEthernetAddress;
  ProtocolType, Operation: WORD; PLEN: BYTE; SenderHA: TEthernetAddress;
  SenderPA: Pointer; TargetHA: TEthernetAddress; TargetPA: Pointer): Boolean;

implementation

const
    TYPELEN_ARP = Swap(Word($0806));

Type
    { ARP Packet }
    PARPPacket = ^TARPPacket;
    TARPPacket = packed record
    	HardwareType: WORD;
        ProtocolType: WORD;
        HLen: BYTE;
        PLen: BYTE;
        Operation: WORD;
        Data: Array[0..50] of BYTE;
    end;

    { This collection contains the protocols currently using ARP }
    PProtocolEntry = ^TProtocolEntry;
    TProtocolEntry = record
        Adapter: TNetworkAdapter;
    	ProtocolType: WORD;
        Routine: TARP_Routine;
    end;

Var
	ProtocolList: PCollection;

procedure ARP_ProcessPacket(Adapter: TNetworkAdapter; Packet: PEthernetPacket);
Var
    i: Integer;
    PE: PProtocolEntry;
    ARPPacket: PARPPacket;
begin
	ARPPacket := @Packet^.Data;
    { Debug }
(*
  	WriteLn('*ARP*');
    WriteLn(' Hardware Type          : ', IntToHex(Swap(ARPPacket^.HardwareType), 4));
    WriteLn(' Protocol Type          : ', IntToHex(Swap(ARPPacket^.ProtocolType), 4));
    WriteLn(' Operation              : ', IntToHex(Swap(ARPPacket^.Operation), 4));
	WriteLn(' HLEN                   : ', ARPPacket^.HLEN);
    WriteLn(' PLEN                   : ', ARPPacket^.PLEN);
*)
    for i := 0 to ProtocolList^.Count - 1 do begin
    	PE := ProtocolList^.At(i);
        if (PE^.Adapter = Adapter) and (PE^.ProtocolType = ARPPacket^.ProtocolType) then begin
            PE^.Routine(
              Adapter,
              Packet^.Header.SourceAddr,
			  ARPPacket^.Operation,
			  ARPPacket^.PLEN,
              PEthernetAddress(@ARPPacket^.Data[0])^,
              @(ARPPacket^.Data[ARPPacket^.HLEN]),
              PEthernetAddress(@ARPPacket^.Data[ARPPacket^.HLEN + ARPPacket^.PLEN])^,
              @(ARPPacket^.Data[ARPPacket^.HLEN + ARPPacket^.PLEN + ARPPacket^.HLEN])
			  );
        end;
    end;
end;

Function ARP_AddProcessor(Adapter: TNetworkAdapter; ProtocolType: WORD; Routine: TARP_Routine): Boolean;
Var
    i: Integer;
    Bound: Boolean;
    PE: PProtocolEntry;
begin
    { If we're not bound to the specified adapter, it's time to do it. }
    Bound := False;
    for i := 0 to ProtocolList^.Count - 1 do begin
        PE := ProtocolList^.At(i);
        if PE^.Adapter = Adapter then begin
            Bound := True;
            Break;
        end;
    end;
    if not Bound and not Adapter.AddProtocol(TYPELEN_ARP,
      @ARP_ProcessPacket) then begin
        ARP_AddProcessor := False;
    	Exit;
    end;
    { Ok. We are bound. Let's move on }
    New(PE);
    PE^.Adapter := Adapter;
    PE^.ProtocolType := ProtocolType;
    PE^.Routine := Routine;
    ProtocolList^.Insert(PE);
    ARP_AddProcessor := True;
end;

function ARP_RemoveProcessor(Adapter: TNetworkAdapter; ProtocolType: WORD): Boolean;
Var
    PE: PProtocolEntry;
    i, Index, Count: integer;
begin
    { Check to find the correct Entry. Also calculate how many other 		}
    { other entries are using this Adapter.                                 }
    Count := 0;
    Index := -1;
    for i := 0 to ProtocolList^.Count - 1 do begin
        PE := ProtocolList^.At(i);
        if (PE^.Adapter = Adapter) then begin
            Inc(Count);
            if (PE^.ProtocolType = ProtocolType) then begin
                Index := i;
                Break;
            end;
        end;
    end;
    { If we didn't found the entry, there's nothing to do.                  }
    if Index = -1 then begin
        ARP_RemoveProcessor := False;
    	Exit;
    end;
    { If this is the only entry using the adapter, we also have to dettach  }
    { from the adapter.                                                     }
    if (Count = 1) and not Adapter.RemoveProtocol(TYPELEN_ARP,
      @ARP_ProcessPacket) then begin
        ARP_RemoveProcessor := False;
        Exit;
    end;
    { All done. Remove the entry.                                           }
    PE := ProtocolList^.At(Index);
    ProtocolList^.Delete(PE);
    Dispose(PE);
    ARP_RemoveProcessor := True;
end;

function ARP_Send(Adapter: TNetworkAdapter; DestEtherAddr: TEthernetAddress;
  ProtocolType, Operation: WORD; PLEN: BYTE; SenderHA: TEthernetAddress;
  SenderPA: Pointer; TargetHA: TEthernetAddress; TargetPA: Pointer): Boolean;
Var
    ARPP: TARPPacket;
	ETP: TEthernetPacket;
	MyEtherAddr: TEthernetAddress;
begin
	{ Armamos el Paquete ARP }
    ARPP.HardwareType := Swap(Word(1));
    ARPP.ProtocolType := ProtocolType;
	ARPP.HLen := 6;
    ARPP.PLen := PLEN;
    ARPP.Operation := Operation;
	Move(SenderHA, ARPP.Data[0], ARPP.HLEN);
    Move(SenderPA^, ARPP.Data[ARPP.HLEN], ARPP.PLEN);
	Move(TargetHA, ARPP.Data[ARPP.HLEN + ARPP.PLEN], ARPP.HLEN);
	Move(TargetPA^, ARPP.Data[ARPP.HLEN + ARPP.PLEN + ARPP.HLEN], ARPP.PLEN);
    { Armamos el Paquete Ethernet }
	Adapter.GetAddress(MyEtherAddr);
	Move(DestEtherAddr, ETP.Header.TargetAddr, SizeOf(TEthernetAddress));
    Move(MyEtherAddr, ETP.Header.SourceAddr, SizeOf(TEthernetAddress));
    ETP.Header.TypeLen := TYPELEN_ARP;
    Move(ARPP, ETP.Data, SizeOf(ARPP));
	{ Enviamos el Paquete Ethernet }
	ARP_Send := Adapter.SendPkt(@ETP, 150);
end;



initialization
    ProtocolList := New(PCollection, Init(10, 1));
finalization
    Dispose(ProtocolList, Done);
end.
