Unit DHCP;

interface

Uses
    SysUtils, Util, Network, IP, Sockets;

Function DHCP_ConfigAdapter(Adapter: TNetworkAdapter; var IPAddress,
  NetMask, DHCPServer: TIPAddress; var Timeout: DWORD): Boolean;
Function DHCP_Release(Adapter: TNetworkAdapter;
  IPAddress, ServerAddress: TIPAddress): Boolean;
Function DHCP_GetInfo(Adapter: TNetworkAdapter;
  IPAddress, DHCPServer: TIPAddress; Option: BYTE;
  var Buffer; var BuffLen: LongInt): Boolean;
Function DHCP_RenewIP(Adapter: TNetworkAdapter; IPAddress,
  ServerAddress: TIPAddress; var Timeout: DWORD): Boolean;

implementation

Type
    TDHCPRec = packed record
        op: BYTE;
        htype: BYTE;
        hlen: BYTE;
        hops: BYTE;
        xid: DWORD;
        secs: WORD;
        flags: WORD;
        ciaddr: DWORD;
        yiaddr: DWORD;
        siaddr: DWORD;
        giaddr: DWORD;
        chaddr: packed array[0..15] of BYTE;
        sname:  packed array[0..63] of CHAR;
        cfile:  packed array[0..127] of CHAR;
        vspec:  packed array[0..1024] of BYTE;
    end;

procedure InitRec(Var Rec: TDHCPRec; op: Integer; HAddr: TEthernetAddress);
begin
    // Set basic Fields
    FillChar(Rec, SizeOf(Rec), 0);
    Rec.op    := op;
    Rec.htype := 1;
    Rec.hlen  := 6;
    Rec.secs  := 0;
    Rec.flags := 0;
    Move(HAddr, Rec.chaddr, 6);
    // Clear Options
    Rec.vspec[0] :=  99;
    Rec.vspec[1] := 130;
    Rec.vspec[2] :=  83;
    Rec.vspec[3] :=  99;
    Rec.vspec[4] := 255;
end;

procedure AddOption(Var Rec: TDHCPRec; OpType: BYTE; Data: AnsiString);
Var
	i, j: Integer;
begin
    for i := High(Rec.vspec) downto 0 do begin
        if Rec.vspec[i] = $FF then begin
            Rec.vspec[i] := OpType;
            Rec.vspec[i + 1] := Length(Data);
            for j := 1 to Length(Data) do begin
             	Rec.vspec[i + j + 1] := BYTE(Data[j]);
            end;
            Rec.vspec[i + Length(Data) + 2] := $FF;
        	Exit;
        end;
    end;
end;

Function ReadOption(Var Rec: TDHCPRec; OpType: BYTE): AnsiString;
Var
    P: PChar;
	i, Len: Integer;
begin
    Result := '';
    P := @Rec.vspec[4];
    Repeat
        if BYTE(P[0]) = 255 then Exit;
        Len := BYTE(P[1]);
        if BYTE(P[0]) = OpType then begin
            for i := 1 to Len do Result := Result + P[1 + i];
            Exit;
        end;
        Inc(P, Len + 2);
    Until False;
end;

Function CalcSize(Var Rec: TDHCPRec): Integer;
Var
	i: Integer;
begin
    Result := SizeOf(TDHCPRec);
    for i := High(Rec.vspec) downto 0 do begin
        if Rec.vspec[i] = $FF then break;
        Dec(Result);
    end;
end;

Function GetClientID(HAddress: TEthernetAddress): AnsiString;
Var
	i: Integer;
begin
    Result := #1;
    for i := 0 to 5 do Result := Result + Chr(HAddress[i]);
end;

Function SendReceive(HSocket: LongInt; Var Rec: TDHCPRec;
  EsperarRespuesta: Boolean): Boolean;
Var
	Timeout: Comp;
	i, ID, RcvLen: DWORD;
begin
	Result := False;
	//LE SACAMOS UNA F
	ID := Random($8FFFFFF);
	Rec.xid := ID;
	for i := 1 to 3 do begin
		if send(HSocket, Rec, CalcSize(Rec), 0) <= 0 then Exit;
		if not EsperarRespuesta then
			Result := True
		else begin                    // 1800 Original
			Timeout := GetMSCounter + 1800;        // 2 segundos
			While GetMSCounter < Timeout do begin
				ioctlsocket(HSocket, FIONREAD, @RcvLen);
				if (RcvLen > 0) then begin
					recv(HSocket, Rec, RcvLen, 0);
					if Rec.xid = ID then begin
(*                      SOpt := ReadOption(Rec, 53);
						if Length(SOpt) > 0 then begin
							Move(SOpt[1], B, 1);
							WriteLn('Respuesta: ', B);
						end else begin
							WriteLn('Sin respuesta');
						end;
*)
						Result := True;
						Break;
	            	end;
    	        end;
            end;
        end;
        if Result then Break;
    end;
end;

Function DHCP_Discover(HAddress: TEthernetAddress;
  var IPAddress, NetMask, ServerAddress: TIPAddress): Boolean;
Var
    SOpt: AnsiString;
    HSocket: Integer;
    ClientRec: TDHCPRec;
    ClientID: AnsiString;
    SockAddr: TInetSockAddr;
begin
    Result := False;
    { Build the Client ID based on the Hardware Address of the Adapter }
    ClientID := GetClientID(HAddress);
    { Create a UDP socket for connecting to the DHCP Server }
    HSocket := socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if HSocket <= 0 then begin
    	Exit;
    end;
    { Bind to the "DHCP Client" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := INADDR_ANY;
    SockAddr.port := Swap(Word(68));
    if not bind(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
    end;
    { Connect to the "DHCP Server" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := INADDR_BROADCAST;
    SockAddr.port := Swap(Word(67));
    if not connect(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
	end;
    { Set-Up a record with the DHCP-Discover information }
    InitRec(ClientRec, 1, HAddress);
    AddOption(ClientRec, 53, #1);               // DHCP Option
    AddOption(ClientRec, 61, ClientID);         // Client identifier

    { Send a DHCP-Discover and wait for the first reply }
    if not SendReceive(HSocket, ClientRec, True) then begin
        // No server replied to our request
	    CloseSocket(HSocket);
        Exit;
    end;
    CloseSocket(HSocket);

    // Tomamos el IP del Server
    SOpt := ReadOption(ClientRec, 54);
    if Length(SOpt) <> 4 then Exit;         // No Server IP ?
    Move(SOpt[1], ServerAddress, 4);
    // Tomamos el NetMask
    SOpt := ReadOption(ClientRec, 1);
    if Length(SOpt) <> 4 then Exit;         // No Netmask ?
    Move(SOpt[1], NetMask, 4);
    // Tomamos el IP
    IPAddress := ClientRec.yiaddr;
	// Listo
    Result := True;
end;


Function DHCP_Request(HAddress: TEthernetAddress; RequestedIP,
  DHCPServer: TIPAddress; Renew: Boolean; var Timeout: DWORD): Boolean;
Var
    SOpt: AnsiString;
    HSocket: Integer;
    ClientRec: TDHCPRec;
    ClientID: AnsiString;
    SockAddr: TInetSockAddr;
begin
    Result := False;

    { Build the Client ID based on the Hardware Address of the Adapter }
    ClientID := GetClientID(HAddress);

    { Create a UDP socket for connecting to the DHCP Server }
    HSocket := socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if HSocket <= 0 then begin
    	Exit;
    end;
    { Bind to the "DHCP Client" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := INADDR_ANY;
    SockAddr.port := Swap(Word(68));
    if not bind(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
    end;

    { Connect to the "DHCP Server" port }
    SockAddr.family := AF_INET;
    if Renew then SockAddr.addr := DHCPServer
      else SockAddr.addr := INADDR_BROADCAST;
    SockAddr.port := Swap(Word(67));
    if not connect(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
	end;

    // Set-Up a record with the DHCP-Request information
	SetLength(SOpt, 4);
    InitRec(ClientRec, 1, HAddress);
    AddOption(ClientRec, 53, #3);			// DHCP Option
    AddOption(ClientRec, 61, ClientID);		// Client Identifier
    AddOption(ClientRec, 55, #51);      // Requested Options
    if Renew then begin
        Clientrec.ciaddr := RequestedIP;
    end else begin
    	Move(DHCPServer,  SOpt[1], 4);		// Server Identifier
    	AddOption(ClientRec, 54, SOpt);
    	Move(RequestedIP, SOpt[1], 4);
        AddOption(ClientRec, 50, SOpt);     // Requested IP Address
    end;


    if not SendReceive(HSocket, ClientRec, True) then begin
        // No server replied our request
        CloseSocket(HSocket);
        Exit;
    end;
    CloseSocket(HSocket);

    { Tomamos el Tiempo Lmite }
    SOpt := ReadOption(ClientRec, 51);
    if Length(SOpt) = 4 then begin
        Move(SOpt[1], Timeout, 4);
        Timeout := SwapDWord(Timeout);
    end;

    { Done }
    Result := True;
end;

Function DHCP_GetInfo(Adapter: TNetworkAdapter;
  IPAddress, DHCPServer: TIPAddress; Option: BYTE;
  var Buffer; var BuffLen: LongInt): Boolean;
Var
    SOpt: AnsiString;
    HSocket: Integer;
    ClientRec: TDHCPRec;
    ClientID: AnsiString;
    SockAddr: TInetSockAddr;
    HAddress: TEthernetAddress;
begin
    Result := False;

    { Build the Client ID based on the Hardware Address of the Adapter }
    Adapter.GetAddress(HAddress);
    ClientID := GetClientID(HAddress);

    { Create a UDP socket for connecting to the DHCP Server }
    HSocket := socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if HSocket <= 0 then begin
    	Exit;
    end;
    { Bind to the "DHCP Client" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := INADDR_ANY;
    SockAddr.port := Swap(Word(68));
    if not bind(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
    end;

    { Connect to the "DHCP Server" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := DHCPServer;
    SockAddr.port := Swap(Word(67));
    if not connect(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
	end;

    // Set-Up a record with the DHCP-Request information
    SetLength(SOpt, 4);
    InitRec(ClientRec, 1, HAddress);
    AddOption(ClientRec, 53, #3);
    AddOption(ClientRec, 61, ClientID);
    Move(DHCPServer,  SOpt[1], 4);
    AddOption(ClientRec, 54, SOpt);
    Move(IPAddress, SOpt[1], 4);
    AddOption(ClientRec, 50, SOpt);
    AddOption(ClientRec, 55, Chr(Option));
    if not SendReceive(HSocket, ClientRec, True) then begin
        // No server replied our request
        CloseSocket(HSocket);
        Exit;
    end;
    CloseSocket(HSocket);

    { Get the Option }
    SOpt := ReadOption(ClientRec, Option);
    if (SOpt <> '') and (BuffLen >= Length(SOpt)) then begin
        BuffLen := Length(SOpt);
        Move(SOpt[1], Buffer, BuffLen);
        Result := True;
    end else begin
    	Result := False;
    end;

end;

Function DHCP_Release(Adapter: TNetworkAdapter;
  IPAddress, ServerAddress: TIPAddress): Boolean;
Var
    SOpt: AnsiString;
    HSocket: Integer;
    ClientRec: TDHCPRec;
    ClientID: AnsiString;
    SockAddr: TInetSockAddr;
    HAddress: TEthernetAddress;
begin
    Result := False;

    { Build the Client ID based on the Hardware Address of the Adapter }
    Adapter.GetAddress(HAddress);
    ClientID := GetClientID(HAddress);

    { Create a UDP socket for connecting to the DHCP Server }
    HSocket := socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if HSocket <= 0 then begin
    	Exit;
    end;

    { Bind to the "DHCP Client" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := INADDR_ANY;
    SockAddr.port := Swap(Word(68));
    if not bind(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
    end;

    { Connect to the "DHCP Server" port }
    SockAddr.family := AF_INET;
    SockAddr.addr := ServerAddress;
    SockAddr.port := Swap(Word(67));
    if not connect(HSocket, SockAddr, SizeOf(SockAddr)) then begin
        CloseSocket(HSocket);
        Exit;
	end;

    // Set-Up a record with the DHCP-Release information
    SetLength(SOpt, 4);
    InitRec(ClientRec, 1, HAddress);
    ClientRec.ciaddr := IPAddress;
    //
    AddOption(ClientRec, 53, #7);			// DHCP Option
    AddOption(ClientRec, 61, ClientID);		// Client Identifier
    Move(ServerAddress,  SOpt[1], 4);
    AddOption(ClientRec, 54, SOpt);			// Server Identifier

    Result := SendReceive(HSocket, ClientRec, False);
    CloseSocket(HSocket);
end;

Function DHCP_ConfigAdapter(Adapter: TNetworkAdapter; var IPAddress,
  NetMask, DHCPServer: TIPAddress; var Timeout: DWORD): Boolean;
Var
    EthAddr: TEthernetAddress;
begin
    Result := False;
    { Get Adapter Hardware Address and build ClientID }
    if not Adapter.GetAddress(EthAddr) then Exit;
    // Send a DCHP-Discover and wait for a reply
    if not DHCP_Discover(EthAddr, IPAddress, NetMask, DHCPServer) then Exit;
    // Send a DHCP-Request and wait for the reply
    if not DHCP_Request(EthAddr, IPAddress, DHCPServer, False, Timeout) then Exit;
	// Done
    Result := True;
end;

Function DHCP_RenewIP(Adapter: TNetworkAdapter; IPAddress,
  ServerAddress: TIPAddress; var Timeout: DWORD): Boolean;
Var
    EthAddr: TEthernetAddress;
begin
	Result := False;
    { Get Adapter Hardware Address and build ClientID }
    if not Adapter.GetAddress(EthAddr) then Exit;

    // Send a DHCP-Request and wait for the reply
    if not DHCP_Request(EthAddr, IPAddress, ServerAddress, True, Timeout) then Exit;

    { Done }
    Result := True;
end;


end.
