Unit ICMP;

interface

Uses
    Network, Sockets, Util, IP;

Function ICMP_Ping(TargetIP: TIPAddress; Timeout: WORD): LongInt;
Function ICMP_Init: Boolean;
Function ICMP_Deinit: Boolean;

implementation

Const
	IPPROTO_ICMP = 1;
    ICMP_PINGREQ = 8;
    ICMP_PINGANS = 0;

Type
	PICMPPingHeader = ^TICMPPingHeader;
	TICMPPingHeader = record
        _Type: BYTE;
        Code: BYTE;
        Checksum: WORD;
        ID: WORD;
        Secuence: WORD;
    end;

Var
	PingAnswer: Boolean;

procedure ProcessIP(SourceIP, TargetIP: TIPAddress;
  Var Data; DataLen: Word);
Var
    Hdr: PICMPPingHeader;
begin
    Hdr := @Data;
    case Hdr^._Type of
    	ICMP_PINGANS:  { Respuesta de PING }
        	begin
                PingAnswer := True;
            end;

		ICMP_PINGREQ:	{ Pedido de Ping }
			begin
			    { Lo contestamos }
			    Hdr^._Type := ICMP_PINGANS;
			    Hdr^.Checksum := 0;
			    Hdr^.Checksum := IP_Checksum(@Hdr^, DataLen, nil, 0);
                IP_Send(SourceIP, IPPROTO_ICMP, Data, Datalen);
            end;
    end;
end;

Function ICMP_Init: Boolean;
begin
	Result := IP_AddProcessor(IPPROTO_ICMP, @ProcessIP);
end;

Function ICMP_Deinit: Boolean;
begin
	ICMP_Deinit := IP_RemoveProcessor(IPPROTO_ICMP);
end;

Function ICMP_Ping(TargetIP: TIPAddress; Timeout: WORD): LongInt;
Const
	PingSecuence: Word = 2200;
    PingData: Array[0..31] of char = 'abcdefghijklmnopqrstuvwabcdefghi';
Type
	TPingRec = packed record
    	Hdr: TICMPPingHeader;
        Data: packed Array[0..31] of char;	{ 32 bytes de datos }
    end;
Var
    Rec: TPingRec;
    Resu: LongInt;
    TimeStop: Comp;
begin
    { Armamos el paquete }
    if PingSecuence < $FFFF then Inc(PingSecuence) else PingSecuence := 0;
    Rec.Hdr._Type    := ICMP_PINGREQ;
	Rec.Hdr.Code     := 0;
    Rec.Hdr.Checksum := 0;
    Rec.Hdr.ID       := 1;
    Rec.Hdr.Secuence := PingSecuence;
    Move(PingData, Rec.Data, SizeOf(Rec.Data));
    Rec.Hdr.Checksum := IP_Checksum(@Rec, SizeOf(Rec), nil, 0);
    { Enviamos el paquete de PING }
	PingAnswer := False;
    resu := IP_Send(TargetIP, IPPROTO_ICMP, Rec, SizeOf(Rec));
    if Resu <> 0 then begin
    	ICMP_Ping := Resu;
        Exit;
    end;
    { Esperamos la respuesta }
    TimeStop := GetMSCounter + Timeout;
    While GetMSCounter < TimeStop do begin
        NET_Work_Low;
        if PingAnswer then begin
            ICMP_Ping := 0;
            Exit;
        end;
    end;
    { Timeout }
    ICMP_Ping := WSAETIMEDOUT;
end;


end.
